@@ -1,5 +1,5 @@
 ###########################################################
-## Demo: Convolution of normal distributions
+## Demo: Convolution of normal distributions   
 ###########################################################
 require(distr)
 
@@ -21,7 +21,8 @@ oldeps <- getdistrOption("TruncQuantile")
 eps <- 1e-8
 distroptions("TruncQuantile" = eps)
 ## support of A1+B1 for FFT convolution is
-## [q(A1)(TruncQuantile), q(B1)(1-TruncQuantile)]
+## [q(A1)(TruncQuantile), 
+##  q(B1)(TruncQuantile, lower.tail = FALSE)]
 
 ## convolution via FFT
 AB1 <- A1+B1
@@ -31,30 +32,31 @@ AB1 <- A1+B1
 #############################
 par(mfrow=c(1,3))
 low <- q(AB)(1e-15)
-upp <- q(AB)(1-1e-15)
+upp <- q(AB)(1e-15, lower.tail = FALSE)
 x <- seq(from = low, to = upp, length = 10000)
 
 ## densities
 plot(x, d(AB)(x), type = "l", lwd = 5)
 lines(x , d(AB1)(x), col = "orange", lwd = 1)
 title("Densities")
-legend(low, d(AB)(5), legend=c("exact", "FFT"), 
+legend("topleft", legend=c("exact", "FFT"), 
         fill=c("black", "orange"))
 
 ## cdfs
 plot(x, p(AB)(x), type = "l", lwd = 5)
 lines(x , p(AB1)(x), col = "orange", lwd = 1)
 title("Cumulative distribution functions")
-legend(low, 1.0, legend=c("exact", "FFT"), 
+legend("topleft", legend=c("exact", "FFT"), 
         fill=c("black", "orange"))
 
 ## quantile functions
 x <- seq(from = eps, to = 1-eps, length = 1000)
-plot(x, q(AB)(x), type = "l", lwd = 5)
+plot("topleft", type = "l", lwd = 5)
 lines(x , q(AB1)(x), col = "orange", lwd = 1) 
 title("Quantile functions")
-legend(0, q(AB)(1-eps), legend=c("exact", "FFT"), 
-        fill=c("black", "orange"))
+legend(0, q(AB)(eps, lower.tail = FALSE), 
+       legend = c("exact", "FFT"), 
+        fill = c("black", "orange"))
 
 ## Since the plots of the results show no 
 ## recognizable differencies, we also compute 
@@ -65,8 +67,9 @@ legend(0, q(AB)(1-eps), legend=c("exact", "FFT"),
 total.var <- function(z, N1, N2){
     0.5*abs(d(N1)(z) - d(N2)(z))
 }
-dv <- integrate(total.var, lower=-Inf, upper=Inf, 
-                rel.tol=1e-8, N1=AB, N2=AB1)
+dv <- integrate(f = total.var, lower = -Inf, 
+                upper = Inf, rel.tol = 1e-8, 
+                N1 = AB, N2 = AB1)
 cat("Total variation distance of densities:\t")
 print(dv) # 4.25e-07
 
